==> s-postgray-0.8.1.tar.gz: s-postgray-0.8.1/include/su/time.h <==
/*@ Time and date utilities.
 *@ TODO C timespec calculations, more constants
 *
 * Copyright (c) 2021 - 2023 Steffen Nurpmeso <steffen@sdaoden.eu>.
 * SPDX-License-Identifier: ISC
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef su_TIME_H
#define su_TIME_H
#include <su/code.h>
#define su_HEADER
#include <su/code-in.h>
C_DECL_BEGIN
struct su_timespec;
/* time {{{ */
/* timespec {{{ */
#define su_TIMESPEC_SEC_MILLIS 1000l
#define su_TIMESPEC_SEC_MICROS 1000000l
#define su_TIMESPEC_SEC_NANOS 1000000000l
struct su_timespec{
	s64 ts_sec;
	sz ts_nano;
};
EXPORT struct su_timespec *su_timespec_current(struct su_timespec *self);
INLINE boole su_timespec_is_valid(struct su_timespec const *self){
	ASSERT_RET(self != NIL, FAL0);
	return (self->ts_sec >= 0 && self->ts_nano >= 0 && self->ts_nano < su_TIMESPEC_SEC_NANOS);
}
INLINE sz su_timespec_cmp(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, -(tp != NIL));
	ASSERT_RET(tp != NIL, 1);
	x = self->ts_sec - tp->ts_sec;
	if(x == 0)
		x = self->ts_nano - tp->ts_nano;
	return (x == 0 ? 0 : x < 0 ? -1 : 1);
}
INLINE boole su_timespec_is_EQ(struct su_timespec const *self, struct su_timespec const *tp){
	ASSERT_RET(self != NIL, (tp == NIL));
	ASSERT_RET(tp != NIL, FAL0);
	return (self->ts_sec == tp->ts_sec && self->ts_nano == tp->ts_nano);
}
INLINE boole su_timespec_is_NE(struct su_timespec const *self, struct su_timespec const *tp){
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	return (self->ts_sec != tp->ts_sec || self->ts_nano != tp->ts_nano);
}
INLINE boole su_timespec_is_LT(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	x = self->ts_sec - tp->ts_sec;
	return (x < 0 || (x == 0 && self->ts_nano < tp->ts_nano));
}
INLINE boole su_timespec_is_LE(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	x = self->ts_sec - tp->ts_sec;
	return (x < 0 || (x == 0 && self->ts_nano <= tp->ts_nano));
}
INLINE boole su_timespec_is_GE(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp == NIL));
	ASSERT_RET(tp != NIL, TRU1);
	x = self->ts_sec - tp->ts_sec;
	return (x > 0 || (x == 0 && self->ts_nano >= tp->ts_nano));
}
INLINE boole su_timespec_is_GT(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, FAL0);
	ASSERT_RET(tp != NIL, TRU1);
	x = self->ts_sec - tp->ts_sec;
	return (x > 0 || (x == 0 && self->ts_nano > tp->ts_nano));
}
INLINE struct su_timespec *su_timespec_add(struct su_timespec *self, struct su_timespec const *tp){
	s64 sec;
	sz nano;
	ASSERT_RET(self != NIL, self);
	ASSERT_RET(tp != NIL, self);
	sec = self->ts_sec;
	nano = self->ts_nano;
	sec += tp->ts_sec;
	nano += tp->ts_nano;
	while(nano >= su_TIMESPEC_SEC_NANOS){
		++sec;
		nano -= su_TIMESPEC_SEC_NANOS;
	}
	self->ts_sec = sec;
	self->ts_nano = nano;
	return self;
}
INLINE struct su_timespec *su_timespec_sub(struct su_timespec *self, struct su_timespec const *tp){
	s64 sec;
	sz nano;
	ASSERT_RET(self != NIL, self);
	ASSERT_RET(tp != NIL, self);
	sec = self->ts_sec;
	nano = self->ts_nano;
	sec -= tp->ts_sec;
	nano -= tp->ts_nano;
	while(nano < 0){
		--sec;
		nano += su_TIMESPEC_SEC_NANOS;
	}
	self->ts_sec = sec;
	self->ts_nano = nano;
	return self;
}
/* }}} */
/* utils {{{ */
#define su_TIME_EPOCH_MAX su_S64_C(0x1D30BE2E1FF)
#define su_TIME_MIN_SECS 60u
#define su_TIME_HOUR_MINS 60u
#define su_TIME_HOUR_SECS (su_TIME_MIN_SECS * su_TIME_HOUR_MINS)
#define su_TIME_DAY_HOURS 24u
#define su_TIME_DAY_MINS (su_TIME_HOUR_MINS * su_TIME_DAY_HOURS)
#define su_TIME_DAY_SECS (su_TIME_HOUR_SECS * su_TIME_DAY_HOURS)
#define su_TIME_YEAR_GREGORIAN 1752u
#define su_TIME_YEAR_DAYS 365u
#define su_TIME_JDN_EPOCH 2440588ul
enum su_time_weekdays{
	su_TIME_WEEKDAY_SUNDAY,
	su_TIME_WEEKDAY_MONDAY,
	su_TIME_WEEKDAY_TUESDAY,
	su_TIME_WEEKDAY_WEDNESDAY,
	su_TIME_WEEKDAY_THURSDAY,
	su_TIME_WEEKDAY_FRIDAY,
	su_TIME_WEEKDAY_SATURDAY
};
#define su_TIME_WEEKDAY_IS_VALID(X) (su_S(uz,X) <= su_TIME_WEEKDAY_SATURDAY)
enum su_time_months{
	su_TIME_MONTH_JANUARY,
	su_TIME_MONTH_FEBRUARY,
	su_TIME_MONTH_MARCH,
	su_TIME_MONTH_APRIL,
	su_TIME_MONTH_MAY,
	su_TIME_MONTH_JUNE,
	su_TIME_MONTH_JULY,
	su_TIME_MONTH_AUGUST,
	su_TIME_MONTH_SEPTEMBER,
	su_TIME_MONTH_OCTOBER,
	su_TIME_MONTH_NOVEMBER,
	su_TIME_MONTH_DECEMBER
};
#define su_TIME_MONTH_IS_VALID(X) (su_S(uz,X) <= su_TIME_MONTH_DECEMBER)
#define su_TIME_WEEKDAY_NAMES_ABBREV_LEN 3u
EXPORT_DATA char const su_time_weekday_names_abbrev[su_TIME_WEEKDAY_SATURDAY +1][su_TIME_WEEKDAY_NAMES_ABBREV_LEN +1];
#define su_TIME_MONTH_NAMES_ABBREV_LEN 3u
EXPORT_DATA char const su_time_month_names_abbrev[su_TIME_MONTH_DECEMBER +1][su_TIME_MONTH_NAMES_ABBREV_LEN +1];
EXPORT boole su_time_epoch_to_gregor(s64 epsec, u32 *yp, u32 *mp, u32 *dp,
		u32 *hourp_or_nil, u32 *minp_or_nil, u32 *secp_or_nil);
EXPORT s64 su_time_gregor_to_epoch(u32 y, u32 m, u32 d, u32 hour, u32 min, u32 sec);
EXPORT uz su_time_gregor_to_jdn(u32 y, u32 m, u32 d);
EXPORT void su_time_jdn_to_gregor(uz jdn, u32 *yp, u32 *mp, u32 *dp);
INLINE boole su_time_year_is_leap(u32 y){
	boole rv = ((y & 3) == 0);
	if(y >= su_TIME_YEAR_GREGORIAN)
		rv = ((rv && y % 100 != 0) || y % 400 == 0);
	return rv;
}
EXPORT uz su_time_msleep(uz millis, boole ignint);
EXPORT s32 su_time_nsleep(struct su_timespec const *dur, struct su_timespec *rem_or_nil);
/* }}} */
/* }}} */
C_DECL_END
#include <su/code-ou.h>
#endif /* su_TIME_H */
/* s-itt-mode */

==> s-postgray-0.8.3.tar.gz: s-postgray-0.8.3/include/su/time.h <==
/*@ Time and date utilities.
 *@ TODO C timespec calculations, more constants
 *
 * Copyright (c) 2021 - 2024 Steffen Nurpmeso <steffen@sdaoden.eu>.
 * SPDX-License-Identifier: ISC
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef su_TIME_H
#define su_TIME_H
#include <su/code.h>
#define su_HEADER
#include <su/code-in.h>
C_DECL_BEGIN
struct su_timespec;
/* time {{{ */
/* timespec {{{ */
#define su_TIMESPEC_SEC_MILLIS 1000l
#define su_TIMESPEC_SEC_MICROS 1000000l
#define su_TIMESPEC_SEC_NANOS 1000000000l
struct su_timespec{
	s64 ts_sec;
	sz ts_nano;
};
EXPORT struct su_timespec *su_timespec_current(struct su_timespec *self);
INLINE boole su_timespec_is_valid(struct su_timespec const *self){
	ASSERT_RET(self != NIL, FAL0);
	return (self->ts_sec >= 0 && self->ts_nano >= 0 && self->ts_nano < su_TIMESPEC_SEC_NANOS);
}
INLINE sz su_timespec_cmp(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, -(tp != NIL));
	ASSERT_RET(tp != NIL, 1);
	x = self->ts_sec - tp->ts_sec;
	if(x == 0)
		x = self->ts_nano - tp->ts_nano;
	return (x == 0 ? 0 : x < 0 ? -1 : 1);
}
INLINE boole su_timespec_is_EQ(struct su_timespec const *self, struct su_timespec const *tp){
	ASSERT_RET(self != NIL, (tp == NIL));
	ASSERT_RET(tp != NIL, FAL0);
	return (self->ts_sec == tp->ts_sec && self->ts_nano == tp->ts_nano);
}
INLINE boole su_timespec_is_NE(struct su_timespec const *self, struct su_timespec const *tp){
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	return (self->ts_sec != tp->ts_sec || self->ts_nano != tp->ts_nano);
}
INLINE boole su_timespec_is_LT(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	x = self->ts_sec - tp->ts_sec;
	return (x < 0 || (x == 0 && self->ts_nano < tp->ts_nano));
}
INLINE boole su_timespec_is_LE(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp != NIL));
	ASSERT_RET(tp != NIL, FAL0);
	x = self->ts_sec - tp->ts_sec;
	return (x < 0 || (x == 0 && self->ts_nano <= tp->ts_nano));
}
INLINE boole su_timespec_is_GE(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, (tp == NIL));
	ASSERT_RET(tp != NIL, TRU1);
	x = self->ts_sec - tp->ts_sec;
	return (x > 0 || (x == 0 && self->ts_nano >= tp->ts_nano));
}
INLINE boole su_timespec_is_GT(struct su_timespec const *self, struct su_timespec const *tp){
	s64 x;
	ASSERT_RET(self != NIL, FAL0);
	ASSERT_RET(tp != NIL, TRU1);
	x = self->ts_sec - tp->ts_sec;
	return (x > 0 || (x == 0 && self->ts_nano > tp->ts_nano));
}
INLINE struct su_timespec *su_timespec_add(struct su_timespec *self, struct su_timespec const *tp){
	s64 sec;
	sz nano;
	ASSERT_RET(self != NIL, self);
	ASSERT_RET(tp != NIL, self);
	sec = self->ts_sec;
	nano = self->ts_nano;
	sec += tp->ts_sec;
	nano += tp->ts_nano;
	while(nano >= su_TIMESPEC_SEC_NANOS){
		++sec;
		nano -= su_TIMESPEC_SEC_NANOS;
	}
	self->ts_sec = sec;
	self->ts_nano = nano;
	return self;
}
INLINE struct su_timespec *su_timespec_sub(struct su_timespec *self, struct su_timespec const *tp){
	s64 sec;
	sz nano;
	ASSERT_RET(self != NIL, self);
	ASSERT_RET(tp != NIL, self);
	sec = self->ts_sec;
	nano = self->ts_nano;
	sec -= tp->ts_sec;
	nano -= tp->ts_nano;
	while(nano < 0){
		--sec;
		nano += su_TIMESPEC_SEC_NANOS;
	}
	self->ts_sec = sec;
	self->ts_nano = nano;
	return self;
}
/* }}} */
/* utils {{{ */
#define su_TIME_EPOCH_MAX su_S64_C(0x1D30BE2E1FF)
#define su_TIME_MIN_SECS 60ul
#define su_TIME_HOUR_MINS 60ul
#define su_TIME_HOUR_SECS (su_TIME_MIN_SECS * su_TIME_HOUR_MINS)
#define su_TIME_DAY_HOURS 24ul
#define su_TIME_DAY_MINS (su_TIME_HOUR_MINS * su_TIME_DAY_HOURS)
#define su_TIME_DAY_SECS (su_TIME_HOUR_SECS * su_TIME_DAY_HOURS)
#define su_TIME_YEAR_GREGORIAN 1752ul
#define su_TIME_YEAR_DAYS 365ul
#define su_TIME_JDN_EPOCH 2440588ul
enum su_time_weekdays{
	su_TIME_WEEKDAY_SUNDAY,
	su_TIME_WEEKDAY_MONDAY,
	su_TIME_WEEKDAY_TUESDAY,
	su_TIME_WEEKDAY_WEDNESDAY,
	su_TIME_WEEKDAY_THURSDAY,
	su_TIME_WEEKDAY_FRIDAY,
	su_TIME_WEEKDAY_SATURDAY
};
#define su_TIME_WEEKDAY_IS_VALID(X) (su_S(uz,X) <= su_TIME_WEEKDAY_SATURDAY)
enum su_time_months{
	su_TIME_MONTH_JANUARY,
	su_TIME_MONTH_FEBRUARY,
	su_TIME_MONTH_MARCH,
	su_TIME_MONTH_APRIL,
	su_TIME_MONTH_MAY,
	su_TIME_MONTH_JUNE,
	su_TIME_MONTH_JULY,
	su_TIME_MONTH_AUGUST,
	su_TIME_MONTH_SEPTEMBER,
	su_TIME_MONTH_OCTOBER,
	su_TIME_MONTH_NOVEMBER,
	su_TIME_MONTH_DECEMBER
};
#define su_TIME_MONTH_IS_VALID(X) (su_S(uz,X) <= su_TIME_MONTH_DECEMBER)
#define su_TIME_WEEKDAY_NAMES_ABBREV_LEN 3u
EXPORT_DATA char const su_time_weekday_names_abbrev[su_TIME_WEEKDAY_SATURDAY +1][su_TIME_WEEKDAY_NAMES_ABBREV_LEN +1];
#define su_TIME_MONTH_NAMES_ABBREV_LEN 3u
EXPORT_DATA char const su_time_month_names_abbrev[su_TIME_MONTH_DECEMBER +1][su_TIME_MONTH_NAMES_ABBREV_LEN +1];
EXPORT boole su_time_epoch_to_gregor(s64 epsec, u32 *yp, u32 *mp, u32 *dp,
		u32 *hourp_or_nil, u32 *minp_or_nil, u32 *secp_or_nil);
EXPORT s64 su_time_gregor_to_epoch(u32 y, u32 m, u32 d, u32 hour, u32 min, u32 sec);
EXPORT uz su_time_gregor_to_jdn(u32 y, u32 m, u32 d);
EXPORT void su_time_jdn_to_gregor(uz jdn, u32 *yp, u32 *mp, u32 *dp);
INLINE boole su_time_year_is_leap(u32 y){
	boole rv = ((y & 3) == 0);
	if(y >= su_TIME_YEAR_GREGORIAN)
		rv = ((rv && y % 100 != 0) || y % 400 == 0);
	return rv;
}
EXPORT uz su_time_msleep(uz millis, boole ignint);
EXPORT s32 su_time_nsleep(struct su_timespec const *dur, struct su_timespec *rem_or_nil);
/* }}} */
/* }}} */
C_DECL_END
#include <su/code-ou.h>
#endif /* su_TIME_H */
/* s-itt-mode */
